@@ -15,6 +15,7 @@ use boolean qw(true false);
 
 use Carp qw(croak);
 use DateTime ();
+use DateTime::HiRes ();
 use DateTime::TimeZone ();
 use List::MoreUtils qw(all any none);
 use Params::Validate ':all';
@@ -23,7 +24,7 @@ use Storable qw(dclone);
 
 use DateTime::Format::Natural::Utils qw(trim);
 
-our $VERSION = '1.12';
+our $VERSION = '1.14';
 
 validation_options(
     on_fail => sub
@@ -216,6 +217,8 @@ sub parse_datetime
 
         $self->_set(%args);
 
+        $self->{datetime}->truncate(to => 'second');
+        $self->_set_truncated;
         $self->_set_valid_exp;
     }
     elsif ($date_string =~ /^([+-]) (\d+?) ([a-zA-Z]+)$/x) {
@@ -252,6 +255,8 @@ sub parse_datetime
 
         $self->_set(%args);
 
+        $self->{datetime}->truncate(to => 'second');
+        $self->_set_truncated;
         $self->_set_valid_exp;
     }
     else {
@@ -304,7 +309,7 @@ sub _parse_init
             $self->{datetime} = dclone($self->{Datetime});
         }
         else {
-            $self->{datetime} = DateTime->$method(
+            $self->{datetime} = DateTime::HiRes->$method(
                 time_zone => $self->{Time_zone},
                 %$args,
             );
@@ -324,6 +329,7 @@ sub _parse_init
     $self->_unset_error;
     $self->_unset_valid_exp;
     $self->_unset_trace;
+    $self->_unset_truncated;
 }
 
 sub parse_datetime_duration
@@ -350,9 +356,9 @@ sub parse_datetime_duration
     }
 
     $self->_pre_duration(\@date_strings);
-    $self->{state} = {};
+    @$self{qw(state truncated_duration)} = ({}, []);
 
-    my (@queue, @traces);
+    my (@queue, @traces, @truncated);
     foreach my $date_string (@date_strings) {
         push @queue, $self->parse_datetime($date_string);
         $self->_save_state(
@@ -363,15 +369,19 @@ sub parse_datetime_duration
         if (@{$self->{traces}}) {
             push @traces, $self->{traces}[0];
         }
+        if ($self->{running_tests}) {
+            push @truncated, $self->_get_truncated;
+        }
     }
 
-    $self->_post_duration(\@queue, \@traces);
+    $self->_post_duration(\@queue, \@traces, \@truncated);
     $self->_restore_state;
 
     delete @$self{qw(duration insert state)};
 
-    @{$self->{traces}} = @traces;
-    $self->{input_string} = $duration_string;
+    @{$self->{traces}}             = @traces;
+    @{$self->{truncated_duration}} = @truncated;
+    $self->{input_string}          = $duration_string;
 
     if ($shrinked) {
         $self->_set_failure;
@@ -534,6 +544,7 @@ sub _truncate
         my $index = $indexes{$unit} - 1;
         if (defined $units[$index] && !exists $self->{modified}{$units[$index]}) {
             $self->{datetime}->truncate(to => $unit);
+            $self->_set_truncated;
             last;
         }
     }
@@ -573,18 +584,36 @@ sub _advance_future
 
     my $now = exists $self->{Datetime}
       ? dclone($self->{Datetime})
-      : DateTime->now(time_zone => $self->{Time_zone});
+      : DateTime::HiRes->now(time_zone => $self->{Time_zone});
 
     my $day_of_week = sub { $_[0]->_Day_of_Week(map $_[0]->{datetime}->$_, qw(year month day)) };
 
-    if ((all { /^(?:second|minute|hour)$/ } keys %modified)
+    my $skip_weekdays = false;
+
+    if ((all { /^(?:(?:nano)?second|minute|hour)$/ } keys %modified)
         && (exists $self->{modified}{hour} && $self->{modified}{hour} == 1)
         && (($self->{Prefer_future} && $self->{datetime} <  $now)
          || ($self->{Demand_future} && $self->{datetime} <= $now))
     ) {
         $self->{postprocess}{day} = 1;
     }
-    elsif ($token_contains->('weekdays_all')
+    elsif (sub {
+        return false unless @{$self->{tokens}} == 2;
+        my ($day, $weekday) = map $self->{data}->__RE($_), qw(day weekday);
+        if ($self->{tokens}->[0] =~ $day
+         && $self->{tokens}->[1] =~ $weekday) {
+            $skip_weekdays = true;
+            return true;
+        }
+        return false;
+    }->()
+        && (all { /^(?:day|month|year)$/ } keys %modified)
+        && (($self->{Prefer_future} && $self->{datetime}->day <  $now->day)
+         || ($self->{Demand_future} && $self->{datetime}->day <= $now->day))
+    ) {
+        $self->{postprocess}{week} = 4;
+    }
+    elsif (($token_contains->('weekdays_all') && !$skip_weekdays)
         && (exists $self->{modified}{day} && $self->{modified}{day} == 1)
         && (($self->{Prefer_future} && $day_of_week->($self) <  $now->wday)
          || ($self->{Demand_future} && $day_of_week->($self) <= $now->wday))
@@ -633,18 +662,23 @@ sub _get_valid_exp   { $_[0]->{valid_expression}         }
 sub _set_valid_exp   { $_[0]->{valid_expression} = true  }
 sub _unset_valid_exp { $_[0]->{valid_expression} = false }
 
+sub _get_truncated   { $_[0]->{truncated}         }
+sub _set_truncated   { $_[0]->{truncated} = true  }
+sub _unset_truncated { $_[0]->{truncated} = false }
+
 sub _get_datetime_object
 {
     my $self = shift;
 
     my $dt = DateTime->new(
-        time_zone => $self->{datetime}->time_zone,
-        year      => $self->{datetime}->year,
-        month     => $self->{datetime}->month,
-        day       => $self->{datetime}->day_of_month,
-        hour      => $self->{datetime}->hour,
-        minute    => $self->{datetime}->minute,
-        second    => $self->{datetime}->second,
+        time_zone  => $self->{datetime}->time_zone,
+        year       => $self->{datetime}->year,
+        month      => $self->{datetime}->month,
+        day        => $self->{datetime}->day_of_month,
+        hour       => $self->{datetime}->hour,
+        minute     => $self->{datetime}->minute,
+        second     => $self->{datetime}->second,
+        nanosecond => $self->{datetime}->nanosecond,
     );
 
     foreach my $unit (keys %{$self->{postprocess}}) {
@@ -924,6 +958,7 @@ valuable suggestions and patches:
  Tim Bunce
  Ricardo Signes
  Felix Ostmann
+ Jrn Clausen
 
 =head1 SEE ALSO
 
